     /**
      * \brief Input Provider Interface
      *
      * This input service module acts as part of the interface
      * for input providers and forwards all input requests to the created 
      * input provider.
      *
      * \author Mihael Schmidt
      * \date   26.12.2010
      *
      * \interface
      */

      *---------------------------------------------------------------------------------------------
      *
      * (C) Copyleft 2010 Mihael Schmidt
      *
      * This file is part of RPG Next Gen Input Providers (RNGIN).
      *
      * RNGIN is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published by
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * RNGIN is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with RNGIN.  If not, see <http://www.gnu.org/licenses/>.
      *
      *---------------------------------------------------------------------------------------------

     H nomain


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D rng_input_check...
     D                 PR
     D   inputProvider...
     D                                 *   const
      *
      /copy 'reflection_h.rpgle'
      /copy 'message_h.rpgle'
      /copy 'input_provider_h.rpgle'


      *-------------------------------------------------------------------------
      * Templates
      *-------------------------------------------------------------------------
      /copy 'input_provider_t.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create input provider
      *
      * Creates the input provider.
      * <br/>
      * The passing of the userdata depends on the input provider implementation.
      * Please look up the API documentation of the implementation.
      * 
      * \param Service program (input provider)
      * \param User data (optional)
      * \param Create procedure name (optional)
      *
      * \return Pointer to input provider or *null if given
      *         service program does not exist 
      * 
      * \throw CPF9898 Input provider does not fully implement the interface
      */
     P rng_input_create...
     P                 B                   export
     D                 PI              *
     D   serviceProgram...
     D                               10A   const
     D   userdata                      *   const options(*nopass)
     D   procedureName...
     D                              256A   const options(*nopass)
      *
     D createProcedureName...
     D                 S            256A
     D procedurePointer...
     D                 S               *   procptr
     D create          PR              *   extproc(procedurePointer)
     D    userdata                     *   const options(*nopass)
      *
     D initProcedureName...
     D                 S            256A
      *
     D inputProvider   S               *
     D inputProviderHeader...
     D                 DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
      /free
       if (%parms() = 3);
         createProcedureName = procedureName;
       else;
         createProcedureName = 'rng_input_create';
       endif;

       procedurePointer = reflection_getProcedurePointer(
                                  serviceProgram :
                                  %trimr(createProcedureName));

       if (procedurePointer <> *null);

         // input provider create/build
         if (%parms() = 1);
           inputProvider = create();
         else;
           inputProvider = create(userData);
         endif;

         rng_input_check(inputProvider);
          
       endif;

       return inputProvider;
      /end-free
     P                 E


     /**
      * \brief Dispose input provider
      *
      * Calls clean up process and frees allocated memory.
      *
      * \param Pointer to input provider
      * 
      * \throws CPF9898 Input provider does not fully implement the interface
      */
     P rng_input_finalize...
     P                 B                   export
     D                 PI
     D  inputProvider...
     D                                 *
      *
     D inputProviderHeader...
     D                 DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
     D procedurePointer...
     D                 S               *   procptr
     D finalize        PR                  extproc(procedurePointer)
     D   inputProvider...
     D                                 *
      /free
       procedurePointer = inputProviderHeader.proc_finalize;

       rng_input_check(inputProvider);

       finalize(inputProvider);

       if (inputProvider <> *null);
         dealloc(n) inputProvider;
       endif;
      /end-free
     P                 E


     /**
      * \brief Open action for input provider
      *
      * Executes the open action for the passed input provider (f. e. opening a file).
      *
      * \param Pointer to input provider
      * 
      * \throws CPF9898 Input provider does not fully implement the interface
      */
     P rng_input_open...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *   const
      *
     D inputProviderHeader...
     D                 DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
      *
     D procedurePointer...
     D                 S               *   procptr
     D open            PR                  extproc(procedurePointer)
     D   inputProvider...
     D                                 *   const
      /free
       procedurePointer = inputProviderHeader.proc_open;

       rng_input_check(inputProvider);

       callp open(inputProvider);
      /end-free
     P                 E


     /**
      * \brief Close action for input provider
      *
      * Executes the close action for the passed input provider (f. e. closing a file).
      *
      * \param Pointer to input provider
      * 
      * \throws CPF9898 Input provider does not fully implement the interface
      */
     P rng_input_close...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *   const
      *
     D inputProviderHeader...
     D                 DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
      *
     D procedurePointer...
     D                 S               *   procptr
     D close           PR                  extproc(procedurePointer)
     D   inputProvider...
     D                                 *   const
      /free
       procedurePointer = inputProviderHeader.proc_close;

       rng_input_check(inputProvider);

       callp close(inputProvider);
      /end-free
     P                 E


     /**
      * \brief Read action for input provider
      *
      * Executes the read action for the passed input provider (f. e. reading a file).
      *
      * \param Pointer to input provider
      * 
      * \throws CPF9898 Input provider does not fully implement the interface
      */
     P rng_input_read...
     P                 B                   export
     D                 PI              N
     D   inputProvider...
     D                                 *   const
     D   data                     65535A   varying
      *
     D inputProviderHeader...
     D                 DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
      *
     D procedurePointer...
     D                 S               *   procptr
     D read            PR              N   extproc(procedurePointer)
     D   inputProvider...
     D                                 *   const
     D   data                     65535A   varying
      /free
       procedurePointer = inputProviderHeader.proc_read;

       rng_input_check(inputProvider);

       return read(inputProvider : data);
      /end-free
     P                 E


     /**
      * \brief Check input provider
      *
      * Checks if the input provider provides procedure pointers for all
      * procedures it should implement.
      *
      * \param Input provider
      *
      * \throws CPF9898 Incomplete implementation
      */
     P rng_input_check...
     P                 B
     D                 PI
     D   inputProvider...
     D                                 *   const
      *
     D header          DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
     D cachedInputProviderId...
     D                 S                   like(rng_input_provider_t.id)
     D                                     static
     D inputProviderChecked...
     D                 S               N   inz(*off) static
      /free
       if (header.id = cachedInputProviderId and
           inputProviderChecked);
         return;
       endif;

       cachedInputProviderId = header.id;

       if (header.proc_create = *null);
         msg_sendEscapeMessage('Input Provider Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       elseif (header.proc_finalize = *null);
         msg_sendEscapeMessage('Input Provider Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       elseif (header.proc_open = *null);
         msg_sendEscapeMessage('Input Provider Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       elseif (header.proc_read = *null);
         msg_sendEscapeMessage('Input Provider Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       elseif (header.proc_close = *null);
         msg_sendEscapeMessage('Input Provider Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       endif;

       inputProviderChecked = *on;
      /end-free
     P                 E
